#include <ESP8266WiFi.h>
#include "SparkFunLSM6DS3.h"
#include "Wire.h"
#include "Protocentral_MAX30205.h"
#include <NTPClient.h>
#include <WiFiUdp.h>

MAX30205 tempSensor;
LSM6DS3 myIMU( I2C_MODE, 0x6B );

// PushingBox scenario DeviceId code and API
String deviceId = "v0091A56A5F45E18";
const char* logServer = "api.pushingbox.com";

//Wifi setup
const char* ssid = "OpenWrt";
const char* password = "workshop123";

WiFiClient client; //potrebno za slanje notifikacije

//Time setup
int16_t utc = +2; //UTC +2:00 Croatia

WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "pool.ntp.org", utc*3600);
int minutesFromBegin;
int initialHour;
int initialMinute;


int initialSetup = 1;     //da posalje poruku da slanje radi
int counter = 0;          //broji ocitanja s akcelerometra
long previousMillis = 0;  //proslo vrijeme, potrebno da odredimo 6 sekundi razlike
long currentMillis; 
long microphoneActivationTime = 0;

float xAxis[20], yAxis[20], zAxis[20]; //podaci o x, y i z ocitanjima

//podaci o maksimalnim x, y i z, te razlici
float maxX, minX, xAxisMovement; 
float maxY, minY, yAxisMovement;
float maxZ, minZ, zAxisMovement;

int movement = 0;   //pokret, detektiran kao usporedba sume i razine koju smatramo pokretom

float movementMinLevel = 0.1; //potrebno odrediti
float fallMinimum = 2.5;
float fromLast[2];
float fromLastHolder[2];
int initalFallDetect = 1;


int fallInterrupt = 0; 

int microphoneInterrupt = 0;
int microphoneDebounce = 0;

int temperatureDetect = 0;
int temperatureCounter = 0; //brojac kojim odredujemo 15 min kako bi svakih 15 min ocitali temperaturu
int tempLevel; //da izbjegnemo prebacivanje floata u char u c-u napravio sam levele
int lastTempLevel = 1;
int tempChange = 0;

int movementCounter = 0; //brojac kojim odredujemo 4h, kako bi odredili broj pokreta u ta 4h
int inactivityMessage = 0;
int movementMin = 20;    //po defaultu je noc, baka spava

void ICACHE_RAM_ATTR handleInterrupt() //interupt rutina
{
  //Serial.println("Detektiran interrupt");
  int microphoneDetect = digitalRead(13);
  if (microphoneDetect) {
   //Serial.println("Microphone interrupt");
   microphoneInterrupt = 1;
   microphoneActivationTime = millis();
   microphoneDebounce = 1;
  }
}


void sendNotification(String message){

  if(WiFi.status() != WL_CONNECTED){
     WiFi.begin(ssid, password);
   }

  Serial.println("- connecting to pushing server: " + String(logServer));
  if (client.connect(logServer, 80)) {
    Serial.println("- succesfully connected");
    
    String postStr = "devid=";
    postStr += String(deviceId);
    postStr += "&message_parameter=";
    postStr += String(message);
    postStr += "\r\n\r\n";
    
    Serial.println("- sending data...");
    
    client.print("POST /pushingbox HTTP/1.1\n");
    client.print("Host: api.pushingbox.com\n");
    client.print("Connection: close\n");
    client.print("Content-Type: application/x-www-form-urlencoded\n");
    client.print("Content-Length: ");
    client.print(postStr.length());
    client.print("\n\n");
    client.print(postStr);
  }
  client.stop();
  Serial.println("- stopping the client");
}


int fallDetection(float x[],float y[],float z[]){
  int sizeOfData = 20;
  
  float sumMinimum;
  float sumMaximum;
  int fallDetected = 1;
  int movementDetected = 2;
  int nothingDetected = 0;
  movementCounter++;
  for (int k=0; k < sizeOfData; k++){
    float sum1 = pow(x[k],2) + pow(y[k],2) + pow(z[k],2);
    
    float sum2 = sqrt(sum1);
      
   
    if(k == 0){
    sumMinimum = sum2;
    sumMaximum = sum2;
    }
    if(sum2 > sumMaximum) sumMaximum = sum2;
    if(sum2 < sumMinimum) sumMinimum = sum2;
    if(k == 18) fromLastHolder[0] = sum2;
    if(k == 19) fromLastHolder[1] = sum2;
  
    if(initalFallDetect){
      initalFallDetect = 0;
    } else {
        if(fromLast[0] < sumMinimum) sumMinimum = fromLast[0];
        if(fromLast[1] < sumMinimum) sumMinimum = fromLast[1];
    }
 }

    fromLast[0] = fromLastHolder[0];
    fromLast[1] = fromLastHolder[1];
    if((sumMaximum - sumMinimum) >= fallMinimum ) return fallDetected;
    if((sumMaximum - sumMinimum) >= movementMinLevel ){ 
    Serial.print("Iz funkcije: kolka razlika suma: ");
    Serial.println(sumMinimum);
    Serial.println(sumMaximum);
    return movementDetected; }
    else return nothingDetected;

}


void setup() {
  Serial.begin(115200);
  delay(10);
  pinMode(13, INPUT_PULLUP);      // Setup notification button on pin D7
  attachInterrupt(digitalPinToInterrupt(13), handleInterrupt, RISING);      // Attach pin 2 interrupt to our handler
  Serial.println("- connecting to Home Router SID: " + String(ssid));
  
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
     delay(500);
     Serial.print(".");
  }
  
  Serial.println();
  Serial.println("- succesfully connected");
  Serial.println("- starting client");
  
  myIMU.begin();        //pokretanje ackelerometra
  tempSensor.begin();   //pokretanje toplomjer
  timeClient.begin();   //pokretanje real-time servera dohvat

  timeClient.update();
  Serial.println(timeClient.getHours());        //ovo daje string, ne int
  initialHour = timeClient.getHours();      //pretvaranje u int, initialHour - sat u kojem je pokrenut program
  initialMinute = timeClient.getMinutes();
  Serial.println(initialHour);
  Serial.println(initialMinute);

  if (initialHour > 6 && initialHour < 21){       //baka spava između 21 i 6 h
    movementMin = 40;
  } else {
    movementMin = 20;
  }
  
}


void loop() {

  if(initialSetup == 1){              //inicijalna poruka prilikom pokretanja
    //Serial.println("Provjera slanja notifikacija");
    //sendNotification("Initial setup");
    initialSetup = 0;
  }
    
  currentMillis = millis();
  if( (unsigned long)(currentMillis - previousMillis) >= 50){ //ako je proslo 20ms ili manje - za uzimanje uzoraka akcelerometra na priblizno 50hz
    xAxis[counter] = myIMU.readFloatAccelX();
    yAxis[counter] = myIMU.readFloatAccelY();
    zAxis[counter] = myIMU.readFloatAccelZ();
     
    previousMillis = currentMillis;
    counter++;
  }
    
  if(microphoneDebounce == 1 && (unsigned long)(millis() - microphoneActivationTime) >= 4000) microphoneDebounce = 0;
    
  if(counter == 20){          //detekcija pada ------ stvari koje se rade svaku 1 sek
    
    counter++;   
    temperatureCounter++;
    fallInterrupt = fallDetection(xAxis, yAxis, zAxis);
    /*Serial.print("Je li pad detektiran? 0-nema pada, 1-pad 2-pokret: ");
    Serial.println(fallInterrupt);
    if(fallInterrupt == 1) Serial.println("Paoooooooo sammm"); */
    counter = 0;
    Serial.print("fallInterrupt: ");
    Serial.println(fallInterrupt);

      if(fallInterrupt == 2){     //detekcija dovoljno pokreta
        Serial.println("Pokret!");
        movement++;
        Serial.print("Broj pokreta: ");
        Serial.println(movement);
        Serial.print ("movementCounter je: ");
        Serial.println(movementCounter);
       
      } 
  }

   if(movementCounter == 14400 && movement < movementMin){
          inactivityMessage == 1;
          movementCounter = 0;
          movement = 0;
   }
   if(movementCounter < 14400 && movement >= movementMin){
          Serial.println("Resetiran brojac pokreta!");
          movementCounter = 0;
          movement = 0;
   }

  if(temperatureCounter == 60){           //detekcija temperature ------ stvari koje se rade svakih 1 min 
    Serial.println("Očitavam temperaturu svake minute..");
    float temp = tempSensor.getTemperature();
    Serial.print("Temperatura tijela je: ");
    Serial.println(temp);
    if(temp < 36){ 
        temperatureDetect = 1;
        tempLevel = 0;
    } else if(temp >= 36 && temp <= 36.9){
        tempLevel = 1;
        temperatureDetect == 1;
    } else if(temp >= 37 && temp <= 37.9){
        temperatureDetect = 1;
        tempLevel = 2;
    } else if(temp >= 38 && temp <= 38.9){
        temperatureDetect = 1;
        tempLevel = 3;
    } else if(temp >= 39){
        temperatureDetect = 1;
        tempLevel = 4;
    }
     
    temperatureCounter = 0;

    minutesFromBegin = initialHour*60 + initialMinute + (millis()/60);
    if ( (minutesFromBegin/60) > 6 && (minutesFromBegin/60) < 21 ){
      movementMin = 40;         //baka budna
    } else {
       movementMin = 20;        //baka spava
    }

    Serial.print("Minimalan broj bakinih pokreta: ------------ ");
    Serial.println(movementMin); 
  }

   
   if(fallInterrupt == 1 || microphoneInterrupt == 1 || temperatureDetect == 1 || inactivityMessage == 1){      //slanje notifikacija
      if(fallInterrupt){
        //sendNotification("Fall detected");
      }
      
      if(microphoneInterrupt && microphoneDebounce == 0){
        //sendNotification("Noise level above threshold"); 
        }

      if(temperatureDetect){
        if(lastTempLevel != tempLevel){ 
          tempChange = 1;
          }
        else tempChange = 0;
        if(tempLevel == 0 && tempChange == 1){
          //sendNotification("Body temperature below 36 degrees Celsius");
          }
        else if(tempLevel == 1 && tempChange == 1){
          //sendNotification("Body temperature between 36 and 36.9 degrees Celsius");
          }
        else if(tempLevel == 2 && tempChange == 1) {
          //sendNotification("Body temperature between 37 and 37.9 degrees Celsius");
          }
        else if(tempLevel == 3 && tempChange == 1){
          //sendNotification("Body temperature between 38 and 38.9 degrees Celsius");
          }
        else if(tempLevel == 4 && tempChange == 1){
          //sendNotification("Body temperature above 39 degrees Celsius");
          }
          lastTempLevel = tempLevel;
       }

      if(inactivityMessage == 1){
        //sendNotification("User inactive for 4 hours!");
        }
      
      fallInterrupt = 0;
      microphoneInterrupt = 0;
      temperatureDetect = 0;
      inactivityMessage = 0;
   }
   
}
